#!/usr/bin/env python
#
# Public Domain 2014-2015 MongoDB, Inc.
# Public Domain 2008-2014 WiredTiger, Inc.
#
# This is free and unencumbered software released into the public domain.
#
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
#
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import os
from suite_subprocess import suite_subprocess
from wtscenario import multiply_scenarios, number_scenarios
import wiredtiger, wttest

# test_huffman02.py
#    Huffman key and value configurations with bad settings
class test_huffman02(wttest.WiredTigerTestCase, suite_subprocess):
    """
    Test basic operations
    """
    table_name = 'table:test_huff'

    huffkey = [
        ('none', dict(huffkey=',huffman_key=none')),
        ('english', dict(huffkey=',huffman_key=english')),
        ('bad', dict(huffkey=',huffman_key=bad')),
    ]
    huffval = [
        ('bad', dict(huffval=',huffman_value=bad')),
    ]
    type = [
        ('file', dict(type='type=file')),
        ('lsm', dict(type='type=lsm')),
    ]
    scenarios = number_scenarios(multiply_scenarios('.',type,huffkey, huffval))

    def test_huffman(self):
        gotException = False
        expectMessage = 'nvalid argument'
        config=self.type + self.huffkey + self.huffval
        with self.expectedStderrPattern(expectMessage):
            try:
                self.pr('expect an error message...')
                self.session.create(self.table_name, config)
            except wiredtiger.WiredTigerError as e:
                gotException = True
                self.pr('got expected exception: ' + str(e))
                self.assertTrue(str(e).find('nvalid argument') >= 0)
        self.assertTrue(gotException, 'expected exception')

if __name__ == '__main__':
    wttest.run()
